/****************************************************************************
* @file     can.h
* @version  V1.00
* $Revision: 1 $
* $Date: 15/06/11 2:44p $
* @brief    NUC970 CAN driver header file
*
* @note
* Copyright (C) 2015 Nuvoton Technology Corp. All rights reserved.
*****************************************************************************/

#ifndef		__NUC97X_CAN_H__
#define 	__NUC97X_CAN_H__


#include "nuc97x.h"
#include <drivers/can.h>

#ifdef  __cplusplus
extern "C"
{
#endif

/** @addtogroup NUC970_Device_Driver NUC970 Device Driver
  @{
*/

/** @addtogroup NUC970_CAN_Driver CAN Driver
  @{
*/

/** @addtogroup NUC970_CAN_EXPORTED_CONSTANTS CAN Exported Constants
  @{
*/

#define RETRY_COUNTS    (0x10000000)

#define TSEG1_MIN 2
#define TSEG1_MAX 16
#define TSEG2_MIN 1
#define TSEG2_MAX 8
#define BRP_MIN   1
#define BRP_MAX   1024  /* 6-bit BRP field + 4-bit BRPE field*/
#define SJW_MAX   4
#define BRP_INC   1
/*
enum reg1{
	C_CAN_CTRL_REG = 0,						
	C_CAN_STS_REG,                          
	C_CAN_ERR_CNT_REG,                      
	C_CAN_BTR_REG,                          
	C_CAN_INT_REG,                          
	C_CAN_TEST_REG,                         
	C_CAN_BRPEXT_REG,                       
	C_CAN_IF1_COMREQ_REG,                   
	C_CAN_IF1_COMMSK_REG,                   
	C_CAN_IF1_MASK1_REG,                    
	C_CAN_IF1_MASK2_REG,                    
	C_CAN_IF1_ARB1_REG,                     
	C_CAN_IF1_ARB2_REG,                     
	C_CAN_IF1_MSGCTRL_REG,                  
	C_CAN_IF1_DATA1_REG,                    
	C_CAN_IF1_DATA2_REG,                    
	C_CAN_IF1_DATA3_REG,                    
	C_CAN_IF1_DATA4_REG,                    
	C_CAN_IF2_COMREQ_REG,                   
	C_CAN_IF2_COMMSK_REG,                   
	C_CAN_IF2_MASK1_REG,                    
	C_CAN_IF2_MASK2_REG,                    
	C_CAN_IF2_ARB1_REG,                     
	C_CAN_IF2_ARB2_REG,                     
	C_CAN_IF2_MSGCTRL_REG,                  
	C_CAN_IF2_DATA1_REG,                    
	C_CAN_IF2_DATA2_REG,                    
	C_CAN_IF2_DATA3_REG,                    
	C_CAN_IF2_DATA4_REG,                    
	C_CAN_TXRQST1_REG,                      
	C_CAN_TXRQST2_REG,                      
	C_CAN_NEWDAT1_REG,                      
	C_CAN_NEWDAT2_REG,                      
	C_CAN_INTPND1_REG,                      
	C_CAN_INTPND2_REG,                      
	C_CAN_MSGVAL1_REG,                      
	C_CAN_MSGVAL2_REG,                      
};                                          
 */
enum reg2{
 REG_CON =0,
 REG_STATUS    ,
 REG_ERR       ,
 REG_BTIME     ,
 REG_IIDR      ,
 REG_TEST      ,
 REG_BRPE      ,
 REG_IF1_CREQ  ,
 REG_IF1_CMASK ,
 REG_IF1_MASK1 ,
 REG_IF1_MASK2 ,
 REG_IF1_ARB1  ,
 REG_IF1_ARB2  ,
 REG_IF1_MCON  ,
 REG_IF1_DAT_A1,
 REG_IF1_DAT_A2,
 REG_IF1_DAT_B1,
 REG_IF1_DAT_B2,
 REG_IF2_CREQ  ,
 REG_IF2_CMASK ,
 REG_IF2_MASK1 ,
 REG_IF2_MASK2 ,
 REG_IF2_ARB1  ,
 REG_IF2_ARB2  ,
 REG_IF2_MCON  ,
 REG_IF2_DAT_A1,
 REG_IF2_DAT_A2,
 REG_IF2_DAT_B1,
 REG_IF2_DAT_B2,
 REG_TXREQ1    ,
 REG_TXREQ2    ,
 REG_NDAT1     ,
 REG_NDAT2     ,
 REG_IPND1     ,
 REG_IPND2     ,
 REG_MVLD1     ,
 REG_MVLD2     ,
 REG_WU_EN     ,
 REG_WU_STATUS ,
};
/*
static const u16 reg_map_c_can[] = {
	[C_CAN_CTRL_REG]	= 0x00,
	[C_CAN_STS_REG]		= 0x02,
	[C_CAN_ERR_CNT_REG]	= 0x04,
	[C_CAN_BTR_REG]		= 0x06,
	[C_CAN_INT_REG]		= 0x08,
	[C_CAN_TEST_REG]	= 0x0A,
	[C_CAN_BRPEXT_REG]	= 0x0C,
	[C_CAN_IF1_COMREQ_REG]	= 0x10,
	[C_CAN_IF1_COMMSK_REG]	= 0x12,
	[C_CAN_IF1_MASK1_REG]	= 0x14,
	[C_CAN_IF1_MASK2_REG]	= 0x16,
	[C_CAN_IF1_ARB1_REG]	= 0x18,
	[C_CAN_IF1_ARB2_REG]	= 0x1A,
	[C_CAN_IF1_MSGCTRL_REG]	= 0x1C,
	[C_CAN_IF1_DATA1_REG]	= 0x1E,
	[C_CAN_IF1_DATA2_REG]	= 0x20,
	[C_CAN_IF1_DATA3_REG]	= 0x22,
	[C_CAN_IF1_DATA4_REG]	= 0x24,
	[C_CAN_IF2_COMREQ_REG]	= 0x40,
	[C_CAN_IF2_COMMSK_REG]	= 0x42,
	[C_CAN_IF2_MASK1_REG]	= 0x44,
	[C_CAN_IF2_MASK2_REG]	= 0x46,
	[C_CAN_IF2_ARB1_REG]	= 0x48,
	[C_CAN_IF2_ARB2_REG]	= 0x4A,
	[C_CAN_IF2_MSGCTRL_REG]	= 0x4C,
	[C_CAN_IF2_DATA1_REG]	= 0x4E,
	[C_CAN_IF2_DATA2_REG]	= 0x50,
	[C_CAN_IF2_DATA3_REG]	= 0x52,
	[C_CAN_IF2_DATA4_REG]	= 0x54,
	[C_CAN_TXRQST1_REG]	= 0x80,
	[C_CAN_TXRQST2_REG]	= 0x82,
	[C_CAN_NEWDAT1_REG]	= 0x90,
	[C_CAN_NEWDAT2_REG]	= 0x92,
	[C_CAN_INTPND1_REG]	= 0xA0,
	[C_CAN_INTPND2_REG]	= 0xA2,
	[C_CAN_MSGVAL1_REG]	= 0xB0,
	[C_CAN_MSGVAL2_REG]	= 0xB2,
};
*/

static const rt_int16_t reg_map_c_can[] = {
	[ REG_CON       ]= 0x00,
	[ REG_STATUS    ]= 0x02,
	[ REG_ERR       ]= 0x04,
	[ REG_BTIME     ]= 0x06,
	[ REG_IIDR      ]= 0x08,
	[ REG_TEST      ]= 0x0A,
	[ REG_BRPE      ]= 0x0C,
	[ REG_IF1_CREQ  ]= 0x10,
	[ REG_IF1_CMASK ]= 0x12,
	[ REG_IF1_MASK1 ]= 0x14,
	[ REG_IF1_MASK2 ]= 0x16,
	[ REG_IF1_ARB1  ]= 0x18,
	[ REG_IF1_ARB2  ]= 0x1A,
	[ REG_IF1_MCON  ]= 0x1C,
	[ REG_IF1_DAT_A1]= 0x1E,
	[ REG_IF1_DAT_A2]= 0x20,
	[ REG_IF1_DAT_B1]= 0x22,
	[ REG_IF1_DAT_B2]= 0x24,
	[ REG_IF2_CREQ  ]= 0x40,
	[ REG_IF2_CMASK ]= 0x42,
	[ REG_IF2_MASK1 ]= 0x44,
	[ REG_IF2_MASK2 ]= 0x46,
	[ REG_IF2_ARB1  ]= 0x48,
	[ REG_IF2_ARB2  ]= 0x4A,
	[ REG_IF2_MCON  ]= 0x4C,
	[ REG_IF2_DAT_A1]= 0x4E,
	[ REG_IF2_DAT_A2]= 0x50,
	[ REG_IF2_DAT_B1]= 0x52,
	[ REG_IF2_DAT_B2]= 0x54,
	[ REG_TXREQ1    ]= 0x80,
	[ REG_TXREQ2    ]= 0x82,
	[ REG_NDAT1     ]= 0x90,
	[ REG_NDAT2     ]= 0x92,
	[ REG_IPND1     ]= 0xA0,
	[ REG_IPND2     ]= 0xA2,
	[ REG_MVLD1     ]= 0xB0,
	[ REG_MVLD2     ]= 0xB2,
	[ REG_WU_EN     ]= 0xB4,
    [ REG_WU_STATUS ]= 0xB6
	};
/*
static const rt_uint16_t  reg_map_d_can[] = {
	[C_CAN_CTRL_REG]	= 0x00,
	[C_CAN_CTRL_EX_REG]	= 0x02,
	[C_CAN_STS_REG]		= 0x04,
	[C_CAN_ERR_CNT_REG]	= 0x08,
	[C_CAN_BTR_REG]		= 0x0C,
	[C_CAN_BRPEXT_REG]	= 0x0E,
	[C_CAN_INT_REG]		= 0x10,
	[C_CAN_TEST_REG]	= 0x14,
	[C_CAN_TXRQST1_REG]	= 0x88,
	[C_CAN_TXRQST2_REG]	= 0x8A,
	[C_CAN_NEWDAT1_REG]	= 0x9C,
	[C_CAN_NEWDAT2_REG]	= 0x9E,
	[C_CAN_INTPND1_REG]	= 0xB0,
	[C_CAN_INTPND2_REG]	= 0xB2,
	[C_CAN_MSGVAL1_REG]	= 0xC4,
	[C_CAN_MSGVAL2_REG]	= 0xC6,
	[C_CAN_IF1_COMREQ_REG]	= 0x100,
	[C_CAN_IF1_COMMSK_REG]	= 0x102,
	[C_CAN_IF1_MASK1_REG]	= 0x104,
	[C_CAN_IF1_MASK2_REG]	= 0x106,
	[C_CAN_IF1_ARB1_REG]	= 0x108,
	[C_CAN_IF1_ARB2_REG]	= 0x10A,
	[C_CAN_IF1_MSGCTRL_REG]	= 0x10C,
	[C_CAN_IF1_DATA1_REG]	= 0x110,
	[C_CAN_IF1_DATA2_REG]	= 0x112,
	[C_CAN_IF1_DATA3_REG]	= 0x114,
	[C_CAN_IF1_DATA4_REG]	= 0x116,
	[C_CAN_IF2_COMREQ_REG]	= 0x120,
	[C_CAN_IF2_COMMSK_REG]	= 0x122,
	[C_CAN_IF2_MASK1_REG]	= 0x124,
	[C_CAN_IF2_MASK2_REG]	= 0x126,
	[C_CAN_IF2_ARB1_REG]	= 0x128,
	[C_CAN_IF2_ARB2_REG]	= 0x12A,
	[C_CAN_IF2_MSGCTRL_REG]	= 0x12C,
	[C_CAN_IF2_DATA1_REG]	= 0x130,
	[C_CAN_IF2_DATA2_REG]	= 0x132,
	[C_CAN_IF2_DATA3_REG]	= 0x134,
	[C_CAN_IF2_DATA4_REG]	= 0x136,
};
*/

enum c_can_dev_id {
	NUC970_CAN0,
	NUC970_CAN1,
};


/**
  * @brief  CAN Tx message structure definition
  */
typedef struct
{
  rt_uint32_t StdId;    /*!< Specifies the standard identifier.
                          This parameter must be a number between Min_Data = 0 and Max_Data = 0x7FF */

  rt_uint32_t ExtId;    /*!< Specifies the extended identifier.
                          This parameter must be a number between Min_Data = 0 and Max_Data = 0x1FFFFFFF */

  rt_uint32_t IDE;      /*!< Specifies the type of identifier for the message that will be transmitted.
                          This parameter can be a value of @ref CAN_Identifier_Type */

  rt_uint32_t RTR;      /*!< Specifies the type of frame for the message that will be transmitted.
                          This parameter can be a value of @ref CAN_remote_transmission_request */

  rt_uint32_t DLC;      /*!< Specifies the length of the frame that will be transmitted.
                          This parameter must be a number between Min_Data = 0 and Max_Data = 8 */

  rt_uint8_t Data[8];   /*!< Contains the data to be transmitted.
                          This parameter must be a number between Min_Data = 0 and Max_Data = 0xFF */

}CanTxMsgTypeDef;

/**
  * @brief  CAN Rx message structure definition
  */
typedef struct
{
  rt_uint32_t StdId;       /*!< Specifies the standard identifier.
                             This parameter must be a number between Min_Data = 0 and Max_Data = 0x7FF */

  rt_uint32_t ExtId;       /*!< Specifies the extended identifier.
                             This parameter must be a number between Min_Data = 0 and Max_Data = 0x1FFFFFFF */

  rt_uint32_t IDE;         /*!< Specifies the type of identifier for the message that will be received.
                             This parameter can be a value of @ref CAN_Identifier_Type */

  rt_uint32_t RTR;         /*!< Specifies the type of frame for the received message.
                             This parameter can be a value of @ref CAN_remote_transmission_request */

  rt_uint32_t DLC;         /*!< Specifies the length of the frame that will be received.
                             This parameter must be a number between Min_Data = 0 and Max_Data = 8 */

  rt_uint8_t Data[8];      /*!< Contains the data to be received.
                             This parameter must be a number between Min_Data = 0 and Max_Data = 0xFF */

  rt_uint32_t FMI;         /*!< Specifies the index of the filter the message stored in the mailbox passes through.
                             This parameter must be a number between Min_Data = 0 and Max_Data = 0xFF */

  rt_uint32_t FIFONumber;  /*!< Specifies the receive FIFO number.
                             This parameter can be CAN_FIFO0 or CAN_FIFO1 */

}CanRxMsgTypeDef;

/* c_can private data structure */
struct c_can_priv {
	int current_status;
	int last_status;
	rt_uint16_t (*read_reg) (struct c_can_priv *priv, enum reg2 index);
	void (*write_reg)(struct c_can_priv *priv, enum reg2 index, rt_uint16_t val);
	unsigned int base;
	const rt_uint16_t *regs;
	int irqno; /* for request_irq() */
	char *name;
    CanTxMsgTypeDef tx_message;
    CanRxMsgTypeDef rx_message;
	unsigned int tx_next;
	unsigned int tx_echo;
	void *priv;		/* for board-specific data */
	rt_uint16_t irqstatus;
	enum c_can_dev_id type;
	unsigned int instance;

};


struct filter_config2
{
	rt_uint8_t  msgobj;		 //Ӧchannel NO  0-31
	rt_uint16_t u32mcon;	//MCONĴֵ
	struct rt_can_msg * msg; //msg
	rt_uint32_t u32mask;	//MASK1MASK2 ֵ

};
struct filter_config
{
	rt_uint8_t  msgobj;		 //Ӧchannel NO  0-31
	rt_uint8_t ide;
	rt_uint32_t  id;		//
	rt_uint8_t rtr;
	rt_uint8_t u8singleOrFifoLast;//Ƿһ
};


#define CAN_CON_TEST_Pos           7                                    /*!< CAN CON: TEST Position */
#define CAN_CON_TEST_Msk           (1ul << CAN_CON_TEST_Pos)            /*!< CAN CON: TEST Mask     */
#define CAN_MODE_NORMAL			   (0ul << CAN_CON_TEST_Pos) 	/*!< CAN CON: normal mode  */

#define CAN_CON_CCE_Pos            6                                    /*!< CAN CON: CCE Position  */
#define CAN_CON_CCE_Msk            (1ul << CAN_CON_CCE_Pos)             /*!< CAN CON: CCE Mask      */

#define CAN_CON_DAR_Pos            5                                    /*!< CAN CON: DAR Position  */
#define CAN_CON_DAR_Msk            (1ul << CAN_CON_DAR_Pos)             /*!< CAN CON: DAR Mask      */

#define CAN_CON_EIE_Pos            3                                    /*!< CAN CON: EIE Position  */
#define CAN_CON_EIE_Msk            (1ul << CAN_CON_EIE_Pos)             /*!< CAN CON: EIE Mask      */

#define CAN_CON_SIE_Pos            2                                    /*!< CAN CON: SIE Position  */
#define CAN_CON_SIE_Msk            (1ul << CAN_CON_SIE_Pos)             /*!< CAN CON: SIE Mask      */

#define CAN_CON_IE_Pos             1                                    /*!< CAN CON: IE Position   */
#define CAN_CON_IE_Msk             (1ul << CAN_CON_IE_Pos)              /*!< CAN CON: IE Mask       */

#define CAN_CON_INIT_Pos           0                                    /*!< CAN CON: INIT Position */
#define CAN_CON_INIT_Msk           (1ul << CAN_CON_INIT_Pos)            /*!< CAN CON: INIT Mask     */

#define CAN_STATUS_BOFF_Pos        7                                    /*!< CAN STATUS: BOFF Position  */
#define CAN_STATUS_BOFF_Msk        (1ul << CAN_STATUS_BOFF_Pos)         /*!< CAN STATUS: BOFF Mask      */

#define CAN_STATUS_EWARN_Pos       6                                    /*!< CAN STATUS: EWARN Position */
#define CAN_STATUS_EWARN_Msk       (1ul << CAN_STATUS_EWARN_Pos)        /*!< CAN STATUS: EWARN Mask     */

#define CAN_STATUS_EPASS_Pos       5                                    /*!< CAN STATUS: EPASS Position */
#define CAN_STATUS_EPASS_Msk       (1ul << CAN_STATUS_EPASS_Pos)        /*!< CAN STATUS: EPASS Mask     */

#define CAN_STATUS_RXOK_Pos        4                                    /*!< CAN STATUS: RXOK Position  */
#define CAN_STATUS_RXOK_Msk        (1ul << CAN_STATUS_RXOK_Pos)         /*!< CAN STATUS: RXOK Mask      */

#define CAN_STATUS_TXOK_Pos        3                                    /*!< CAN STATUS: TXOK Position  */
#define CAN_STATUS_TXOK_Msk        (1ul << CAN_STATUS_TXOK_Pos)         /*!< CAN STATUS: TXOK Mask      */

#define CAN_STATUS_LEC_Pos         0                                    /*!< CAN STATUS: LEC Position   */
#define CAN_STATUS_LEC_Msk         (0x3ul << CAN_STATUS_LEC_Pos)        /*!< CAN STATUS: LEC Mask       */

#define CAN_ERR_RP_Pos             15                                   /*!< CAN ERR: RP Position       */
#define CAN_ERR_RP_Msk             (1ul << CAN_ERR_RP_Pos)              /*!< CAN ERR: RP Mask           */

#define CAN_ERR_REC_Pos            8                                    /*!< CAN ERR: REC Position      */
#define CAN_ERR_REC_Msk            (0x7Ful << CAN_ERR_REC_Pos)          /*!< CAN ERR: REC Mask          */

#define CAN_ERR_TEC_Pos            0                                    /*!< CAN ERR: TEC Position      */
#define CAN_ERR_TEC_Msk            (0xFFul << CAN_ERR_TEC_Pos)          /*!< CAN ERR: TEC Mask          */

#define CAN_BTIME_TSEG2_Pos        12                                   /*!< CAN BTIME: TSEG2 Position  */
#define CAN_BTIME_TSEG2_Msk        (0x7ul << CAN_BTIME_TSEG2_Pos)       /*!< CAN BTIME: TSEG2 Mask      */

#define CAN_BTIME_TSEG1_Pos        8                                    /*!< CAN BTIME: TSEG1 Position  */
#define CAN_BTIME_TSEG1_Msk        (0xFul << CAN_BTIME_TSEG1_Pos)       /*!< CAN BTIME: TSEG1 Mask      */

#define CAN_BTIME_SJW_Pos          6                                    /*!< CAN BTIME: SJW Position    */
#define CAN_BTIME_SJW_Msk          (0x3ul << CAN_BTIME_SJW_Pos)         /*!< CAN BTIME: SJW Mask        */

#define CAN_BTIME_BRP_Pos          0                                    /*!< CAN BTIME: BRP Position    */
#define CAN_BTIME_BRP_Msk          (0x3Ful << CAN_BTIME_BRP_Pos)        /*!< CAN BTIME: BRP Mask        */

#define CAN_IIDR_INTID_Pos         0                                    /*!< CAN IIDR: INTID Position   */
#define CAN_IIDR_INTID_Msk         (0xFFFFul << CAN_IIDR_INTID_Pos)     /*!< CAN IIDR: INTID Mask       */

#define CAN_TEST_RX_Pos            7                                    /*!< CAN TEST: RX Position      */
#define CAN_TEST_RX_Msk            (1ul << CAN_TEST_RX_Pos)             /*!< CAN TEST: RX Mask          */

#define CAN_TEST_TX_Pos            5                                    /*!< CAN TEST: TX Position      */
#define CAN_TEST_TX_Msk            (0x3ul << CAN_TEST_TX_Pos)           /*!< CAN TEST: TX Mask          */

#define CAN_TEST_LBACK_Pos         4                                    /*!< CAN TEST: LBACK Position   */
#define CAN_TEST_LBACK_Msk         (1ul << CAN_TEST_LBACK_Pos)          /*!< CAN TEST: LBACK Mask       */
#define CAN_MODE_LOOPBACK					 CAN_TEST_LBACK_Msk
#define CAN_TEST_SILENT_Pos        3                                    /*!< CAN TEST: Silent Position  */
#define CAN_TEST_SILENT_Msk        (1ul << CAN_TEST_SILENT_Pos)         /*!< CAN TEST: Silent Mask      */
#define CAN_MODE_SILENT						CAN_TEST_SILENT_Msk
#define CAN_TEST_BASIC_Pos         2                                    /*!< CAN TEST: Basic Position   */
#define CAN_TEST_BASIC_Msk         (1ul << CAN_TEST_BASIC_Pos)          /*!< CAN TEST: Basic Mask       */
#define CAN_MODE_SILENT_LOOPBACK		CAN_MODE_SILENT |CAN_MODE_LOOPBACK
#define CAN_BRPE_BRPE_Pos          0                                    /*!< CAN BRPE: BRPE Position    */
#define CAN_BRPE_BRPE_Msk          (0xFul << CAN_BRPE_BRPE_Pos)         /*!< CAN BRPE: BRPE Mask        */

#define CAN_IF_CREQ_BUSY_Pos       15                                   /*!< CAN IFnCREQ: BUSY Position */
#define CAN_IF_CREQ_BUSY_Msk       (1ul << CAN_IF_CREQ_BUSY_Pos)        /*!< CAN IFnCREQ: BUSY Mask     */

#define CAN_IF_CREQ_MSGNUM_Pos     0                                    /*!< CAN IFnCREQ: MSGNUM Position */
#define CAN_IF_CREQ_MSGNUM_Msk     (0x3Ful << CAN_IF_CREQ_MSGNUM_Pos)   /*!< CAN IFnCREQ: MSGNUM Mask     */

#define CAN_IF_CMASK_WRRD_Pos      7                                    /*!< CAN IFnCMASK: WRRD Position */
#define CAN_IF_CMASK_WRRD_Msk      (1ul << CAN_IF_CMASK_WRRD_Pos)       /*!< CAN IFnCMASK: WRRD Mask     */

#define CAN_IF_CMASK_MASK_Pos      6                                    /*!< CAN IFnCMASK: MASK Position */
#define CAN_IF_CMASK_MASK_Msk      (1ul << CAN_IF_CMASK_MASK_Pos)       /*!< CAN IFnCMASK: MASK Mask     */

#define CAN_IF_CMASK_ARB_Pos       5                                    /*!< CAN IFnCMASK: ARB Position  */
#define CAN_IF_CMASK_ARB_Msk       (1ul << CAN_IF_CMASK_ARB_Pos)        /*!< CAN IFnCMASK: ARB Mask      */

#define CAN_IF_CMASK_CONTROL_Pos   4                                    /*!< CAN IFnCMASK: CONTROL Position */
#define CAN_IF_CMASK_CONTROL_Msk   (1ul << CAN_IF_CMASK_CONTROL_Pos)    /*!< CAN IFnCMASK: CONTROL Mask */

#define CAN_IF_CMASK_CLRINTPND_Pos 3                                    /*!< CAN IFnCMASK: CLRINTPND Position */
#define CAN_IF_CMASK_CLRINTPND_Msk (1ul << CAN_IF_CMASK_CLRINTPND_Pos)  /*!< CAN IFnCMASK: CLRINTPND Mask */

#define CAN_IF_CMASK_TXRQSTNEWDAT_Pos 2                                         /*!< CAN IFnCMASK: TXRQSTNEWDAT Position */
#define CAN_IF_CMASK_TXRQSTNEWDAT_Msk (1ul << CAN_IF_CMASK_TXRQSTNEWDAT_Pos)    /*!< CAN IFnCMASK: TXRQSTNEWDAT Mask     */

#define CAN_IF_CMASK_DATAA_Pos     1                                    /*!< CAN IFnCMASK: DATAA Position */
#define CAN_IF_CMASK_DATAA_Msk     (1ul << CAN_IF_CMASK_DATAA_Pos)      /*!< CAN IFnCMASK: DATAA Mask     */

#define CAN_IF_CMASK_DATAB_Pos     0                                    /*!< CAN IFnCMASK: DATAB Position */
#define CAN_IF_CMASK_DATAB_Msk     (1ul << CAN_IF_CMASK_DATAB_Pos)      /*!< CAN IFnCMASK: DATAB Mask     */

#define CAN_IF_MASK1_MSK_Pos       0                                    /*!< CAN IFnMASK1: MSK Position   */
#define CAN_IF_MASK1_MSK_Msk       (0xFFul << CAN_IF_MASK1_MSK_Pos)     /*!< CAN IFnMASK1: MSK Mask       */

#define CAN_IF_MASK2_MXTD_Pos      15                                   /*!< CAN IFnMASK2: MXTD Position */
#define CAN_IF_MASK2_MXTD_Msk      (1ul << CAN_IF_MASK2_MXTD_Pos)       /*!< CAN IFnMASK2: MXTD Mask     */

#define CAN_IF_MASK2_MDIR_Pos      14                                   /*!< CAN IFnMASK2: MDIR Position */
#define CAN_IF_MASK2_MDIR_Msk      (1ul << CAN_IF_MASK2_MDIR_Pos)       /*!< CAN IFnMASK2: MDIR Mask     */

#define CAN_IF_MASK2_MSK_Pos       0                                    /*!< CAN IFnMASK2: MSK Position */
#define CAN_IF_MASK2_MSK_Msk       (0x1FFul << CAN_IF_MASK2_MSK_Pos)    /*!< CAN IFnMASK2: MSK Mask     */

#define CAN_IF_ARB1_ID_Pos         0                                    /*!< CAN IFnARB1: ID Position   */
#define CAN_IF_ARB1_ID_Msk         (0xFFFFul << CAN_IF_ARB1_ID_Pos)     /*!< CAN IFnARB1: ID Mask       */

#define CAN_IF_ARB2_MSGVAL_Pos     15                                   /*!< CAN IFnARB2: MSGVAL Position */
#define CAN_IF_ARB2_MSGVAL_Msk     (1ul << CAN_IF_ARB2_MSGVAL_Pos)      /*!< CAN IFnARB2: MSGVAL Mask     */

#define CAN_IF_ARB2_XTD_Pos        14                                   /*!< CAN IFnARB2: XTD Position    */
#define CAN_IF_ARB2_XTD_Msk        (1ul << CAN_IF_ARB2_XTD_Pos)         /*!< CAN IFnARB2: XTD Mask        */

#define CAN_IF_ARB2_DIR_Pos        13                                   /*!< CAN IFnARB2: DIR Position    */
#define CAN_IF_ARB2_DIR_Msk        (1ul << CAN_IF_ARB2_DIR_Pos)         /*!< CAN IFnARB2: DIR Mask        */

#define CAN_IF_ARB2_ID_Pos         0                                    /*!< CAN IFnARB2: ID Position     */
#define CAN_IF_ARB2_ID_Msk         (0x1FFFul << CAN_IF_ARB2_ID_Pos)     /*!< CAN IFnARB2: ID Mask         */

#define CAN_IF_MCON_NEWDAT_Pos     15                                   /*!< CAN IFnMCON: NEWDAT Position */
#define CAN_IF_MCON_NEWDAT_Msk     (1ul << CAN_IF_MCON_NEWDAT_Pos)      /*!< CAN IFnMCON: NEWDAT Mask     */

#define CAN_IF_MCON_MSGLST_Pos     14                                   /*!< CAN IFnMCON: MSGLST Position */
#define CAN_IF_MCON_MSGLST_Msk     (1ul << CAN_IF_MCON_MSGLST_Pos)      /*!< CAN IFnMCON: MSGLST Mask     */

#define CAN_IF_MCON_INTPND_Pos     13                                   /*!< CAN IFnMCON: INTPND Position */
#define CAN_IF_MCON_INTPND_Msk     (1ul << CAN_IF_MCON_INTPND_Pos)      /*!< CAN IFnMCON: INTPND Mask     */

#define CAN_IF_MCON_UMASK_Pos      12                                   /*!< CAN IFnMCON: UMASK Position  */
#define CAN_IF_MCON_UMASK_Msk      (1ul << CAN_IF_MCON_UMASK_Pos)       /*!< CAN IFnMCON: UMASK Mask      */

#define CAN_IF_MCON_TXIE_Pos       11                                   /*!< CAN IFnMCON: TXIE Position   */
#define CAN_IF_MCON_TXIE_Msk       (1ul << CAN_IF_MCON_TXIE_Pos)        /*!< CAN IFnMCON: TXIE Mask       */

#define CAN_IF_MCON_RXIE_Pos       10                                   /*!< CAN IFnMCON: RXIE Position   */
#define CAN_IF_MCON_RXIE_Msk       (1ul << CAN_IF_MCON_RXIE_Pos)        /*!< CAN IFnMCON: RXIE Mask       */

#define CAN_IF_MCON_RMTEN_Pos      9                                    /*!< CAN IFnMCON: RMTEN Position  */
#define CAN_IF_MCON_RMTEN_Msk      (1ul << CAN_IF_MCON_RMTEN_Pos)       /*!< CAN IFnMCON: RMTEN Mask      */

#define CAN_IF_MCON_TXRQST_Pos     8                                    /*!< CAN IFnMCON: TXRQST Position */
#define CAN_IF_MCON_TXRQST_Msk     (1ul << CAN_IF_MCON_TXRQST_Pos)      /*!< CAN IFnMCON: TXRQST Mask     */

#define CAN_IF_MCON_EOB_Pos        7                                    /*!< CAN IFnMCON: EOB Position    */
#define CAN_IF_MCON_EOB_Msk        (1ul << CAN_IF_MCON_EOB_Pos)         /*!< CAN IFnMCON: EOB Mask        */

#define CAN_IF_MCON_DLC_Pos        0                                    /*!< CAN IFnMCON: DLC Position    */
#define CAN_IF_MCON_DLC_Msk        (0xFul << CAN_IF_MCON_DLC_Pos)       /*!< CAN IFnMCON: DLC Mask        */

#define CAN_IF_DAT_A1_DATA1_Pos    8                                    /*!< CAN IFnDATAA1: DATA1 Position */
#define CAN_IF_DAT_A1_DATA1_Msk    (0xFFul << CAN_IF_DAT_A1_DATA1_Pos)  /*!< CAN IFnDATAA1: DATA1 Mask     */

#define CAN_IF_DAT_A1_DATA0_Pos    0                                    /*!< CAN IFnDATAA1: DATA0 Position */
#define CAN_IF_DAT_A1_DATA0_Msk    (0xFFul << CAN_IF_DAT_A1_DATA0_Pos)  /*!< CAN IFnDATAA1: DATA0 Mask     */

#define CAN_IF_DAT_A2_DATA3_Pos    8                                    /*!< CAN IFnDATAA1: DATA3 Position */
#define CAN_IF_DAT_A2_DATA3_Msk    (0xFFul << CAN_IF_DAT_A2_DATA3_Pos)  /*!< CAN IFnDATAA1: DATA3 Mask     */

#define CAN_IF_DAT_A2_DATA2_Pos    0                                    /*!< CAN IFnDATAA1: DATA2 Position */
#define CAN_IF_DAT_A2_DATA2_Msk    (0xFFul << CAN_IF_DAT_A2_DATA2_Pos)  /*!< CAN IFnDATAA1: DATA2 Mask     */

#define CAN_IF_DAT_B1_DATA5_Pos    8                                    /*!< CAN IFnDATAB1: DATA5 Position */
#define CAN_IF_DAT_B1_DATA5_Msk    (0xFFul << CAN_IF_DAT_B1_DATA5_Pos)  /*!< CAN IFnDATAB1: DATA5 Mask */

#define CAN_IF_DAT_B1_DATA4_Pos    0                                    /*!< CAN IFnDATAB1: DATA4 Position */
#define CAN_IF_DAT_B1_DATA4_Msk    (0xFFul << CAN_IF_DAT_B1_DATA4_Pos)  /*!< CAN IFnDATAB1: DATA4 Mask */

#define CAN_IF_DAT_B2_DATA7_Pos    8                                    /*!< CAN IFnDATAB2: DATA7 Position */
#define CAN_IF_DAT_B2_DATA7_Msk    (0xFFul << CAN_IF_DAT_B2_DATA7_Pos)  /*!< CAN IFnDATAB2: DATA7 Mask     */

#define CAN_IF_DAT_B2_DATA6_Pos    0                                    /*!< CAN IFnDATAB2: DATA6 Position */
#define CAN_IF_DAT_B2_DATA6_Msk    (0xFFul << CAN_IF_DAT_B2_DATA6_Pos)  /*!< CAN IFnDATAB2: DATA6 Mask     */

#define CAN_IF_TXRQST1_TXRQST_Pos  0                                        /*!< CAN IFnTXRQST1: TXRQST Position */
#define CAN_IF_TXRQST1_TXRQST_Msk  (0xFFFFul << CAN_IF_TXRQST1_TXRQST_Pos)  /*!< CAN IFnTXRQST1: TXRQST Mask     */

#define CAN_IF_TXRQST2_TXRQST_Pos  0                                        /*!< CAN IFnTXRQST2: TXRQST Position  */
#define CAN_IF_TXRQST2_TXRQST_Msk  (0xFFFFul << CAN_IF_TXRQST2_TXRQST_Pos)  /*!< CAN IFnTXRQST2: TXRQST Mask      */

#define CAN_IF_NDAT1_NEWDATA_Pos   0                                        /*!< CAN IFnNDAT1: NEWDATA Position */
#define CAN_IF_NDAT1_NEWDATA_Msk   (0xFFFFul << CAN_IF_NDAT1_NEWDATA_Pos)   /*!< CAN IFnNDAT1: NEWDATA Mask     */

#define CAN_IF_NDAT2_NEWDATA_Pos   0                                        /*!< CAN IFnNDAT2: NEWDATA Position */
#define CAN_IF_NDAT2_NEWDATA_Msk   (0xFFFFul << CAN_IF_NDAT2_NEWDATA_Pos)   /*!< CAN IFnNDAT2: NEWDATA Mask     */

#define CAN_IF_IPND1_INTPND_Pos   0                                         /*!< CAN IFnIPND1: INTPND Position */
#define CAN_IF_IPND1_INTPND_Msk   (0xFFFFul << CAN_IF_IPND1_INTPND_Pos)     /*!< CAN IFnIPND1: INTPND Mask     */

#define CAN_IF_IPND2_INTPND_Pos   0                                         /*!< CAN IFnIPND2: INTPND Position */
#define CAN_IF_IPND2_INTPND_Msk   (0xFFFFul << CAN_IF_IPND2_INTPND_Pos)     /*!< CAN IFnIPND2: INTPND Mask     */

#define CAN_IF_MVLD1_MSGVAL_Pos   0                                         /*!< CAN IFnMVLD1: MSGVAL Position */
#define CAN_IF_MVLD1_MSGVAL_Msk   (0xFFFFul << CAN_IF_MVLD1_MSGVAL_Pos)     /*!< CAN IFnMVLD1: MSGVAL Mask     */

#define CAN_IF_MVLD2_MSGVAL_Pos   0                                         /*!< CAN IFnMVLD2: MSGVAL Position */
#define CAN_IF_MVLD2_MSGVAL_Msk   (0xFFFFul << CAN_IF_MVLD2_MSGVAL_Pos)     /*!< CAN IFnMVLD2: MSGVAL Mask     */

#define CAN_WUEN_WAKUP_EN_Pos     0                                         /*!< CAN WUEN: WAKUP_EN Position */
#define CAN_WUEN_WAKUP_EN_Msk    (1ul << CAN_WUEN_WAKUP_EN_Pos)             /*!< CAN WUEN: WAKUP_EN Mask     */

#define CAN_WUSTATUS_WAKUP_STS_Pos     0                                    /*!< CAN WUSTATUS: WAKUP_STS Position */
#define CAN_WUSTATUS_WAKUP_STS_Msk    (1ul << CAN_WUSTATUS_WAKUP_STS_Pos)   /*!< CAN WUSTATUS: WAKUP_STS Mask     */

/// @cond HIDDEN_SYMBOLS
#define CAN_OFFSET 0x400
/// @endcond HIDDEN_SYMBOLS
	
/** \brief  Message ID types
 */
typedef enum {
    CAN_STD_ID = 0, /*!< Standard Identifier  */
    CAN_EXT_ID = 1  /*!< Extended Identifier  */
} E_CAN_ID_TYPE;

/** \brief  Message Frame types
 */
typedef enum {
    REMOTE_FRAME = 0,  /*!< Remote Frame  */
    DATA_FRAME   = 1   /*!< Data Frame    */
} E_CAN_FRAME_TYPE;

/** \brief  CAN message structure
 */
typedef struct {
    rt_uint32_t  IdType;     /*!< Identifier Type     */
    rt_uint32_t  FrameType;  /*!< Frame Type          */
    rt_uint32_t  Id;         /*!< Message Identifier  */
    rt_uint8_t   DLC;        /*!< Data Length Code    */
    rt_uint8_t   Data[8];    /*!< Data byte 0 ~ 7     */
} STR_CANMSG_T;

/** \brief  CAN mask message structure
 */
typedef struct {
    rt_uint8_t   u8Xtd;     /*!< Extended Identifier  */
    rt_uint8_t   u8Dir;     /*!< Message Direction    */
    rt_uint32_t  u32Id;     /*!< Message Identifier   */
    rt_uint8_t   u8IdType;  /*!< Identifier Type      */
} STR_CANMASK_T;

/** \brief  CAN operation mode: normal/basic mode
 */
typedef enum {
    CAN_NORMAL_MODE = 1, /*!< Normal Mode  */
    CAN_BASIC_MODE = 2   /*!< Basic Mode   */
} CAN_MODE_SELECT;

#define ALL_MSG  32  /*!< All Message ram number   */
#define MSG(id)  id  /*!< Message ram number       */


/*@}*/ /* end of group NUC970_CAN_EXPORTED_CONSTANTS */


/** @addtogroup NUC970_CAN_EXPORTED_FUNCTIONS CAN Exported Functions
  @{
*/

/**
 *  @brief    Get interrupt status
 *
 *  @param[in]    can  CAN channel
 *
 *  @return   CAN module status register value
 *  \hideinitializer
 */
#define CAN_GET_INT_STATUS(can)    inpw(REG_CAN0_STATUS+(can*CAN_OFFSET))  //(can->STATUS)

/**
 *  @brief    Get specified interrupt pending status
 *
 *  @param[in]    can  CAN channel
 *
 *  @return   The source of the interrupt.
 *  \hideinitializer 
 */
#define CAN_GET_INT_PENDING_STATUS(can)     inpw(REG_CAN0_IIDR+(can*CAN_OFFSET))  //(can->IIDR)

/**
 *  @brief    Disable Wakeup function
 *
 *  @param[in]    can  CAN channel
 *
 *  @return   None
 * \hideinitializer 
 */
#define CAN_DISABLE_WAKEUP(can)             outpw((REG_CAN0_WU_EN+(can*CAN_OFFSET)), 0x0)  //(can->WU_IE = 0)

/**
 *  @brief    Enable Wakeup function
 *
 *  @param[in]    can  CAN channel
 *
 *  @return   None
 * \hideinitializer 
 */
#define CAN_ENABLE_WAKEUP(can)              outpw((REG_CAN0_WU_EN+(can*CAN_OFFSET)), CAN_WUEN_WAKUP_EN_Msk)  //(can->WU_IE = CAN_WUEN_WAKUP_EN_Msk)

/**
 *  @brief    Get specified Message Object new data into bit value
 *
 *  @param[in]    can  CAN channel
 *  @param[in]    u32MsgNum  Specified Message Object number. (0 ~ 31)
 *
 *  @return   Specified Message Object new data into bit value.
 * \hideinitializer 
 */
#define CAN_GET_NEW_DATA_IN_BIT(can, u32MsgNum)    (u32MsgNum < 16 ? inpw(REG_CAN0_NDAT1+(can*CAN_OFFSET)) & (1 << u32MsgNum) : inpw(REG_CAN0_NDAT2+(can*CAN_OFFSET)) & (1 << (u32MsgNum-16))) 


/*---------------------------------------------------------------------------------------------------------*/
/* Define CAN functions prototype                                                                          */
/*---------------------------------------------------------------------------------------------------------*/
rt_uint32_t CAN_SetBaudRate(struct c_can_priv *can_priv, rt_uint32_t u32BaudRate);
rt_uint32_t CAN_GetCANBitRate(struct c_can_priv * can_priv);
rt_uint32_t CAN_IsNewDataReceived(struct c_can_priv *can_priv, rt_uint8_t u8MsgObj);
rt_err_t CAN_IsTxRqstPending(struct c_can_priv *can_priv,rt_uint8_t u8MsgObj);
rt_int32_t CAN_ReadMsgObj(struct c_can_priv * can_priv, rt_uint8_t u8MsgObj, rt_uint8_t u8Release, struct rt_can_msg* pCanMsg);
void CAN_EnterInitMode(struct c_can_priv *  can_priv);rt_uint32_t CAN_Open(struct c_can_priv *can_priv, rt_uint32_t u32BaudRate, rt_uint32_t u32Mode);
rt_err_t CAN_Transmit(struct c_can_priv *can_priv, rt_uint32_t u32MsgNum , struct rt_can_msg* pCanMsg);
rt_int32_t CAN_Receive(struct c_can_priv *can_priv, rt_uint32_t u32MsgNum ,  struct  rt_can_msg* pCanMsg);
void CAN_CLR_INT_PENDING_BIT(struct c_can_priv *can_priv, rt_uint8_t u32MsgNum);
void CAN_EnableInt(struct c_can_priv *can_priv, rt_uint32_t u32Mask);
void CAN_DisableInt(struct c_can_priv *can_priv, rt_uint32_t u32Mask);
rt_int32_t CAN_SetMultiRxMsg(struct c_can_priv *can_priv, rt_uint32_t u32MsgNum ,rt_uint32_t u32MsgCount, rt_uint32_t u32IDType,rt_uint8_t rtr , rt_uint32_t u32ID);
rt_int32_t CAN_SetRxMsg(struct c_can_priv *can_priv, rt_uint32_t u32MsgNum , rt_uint32_t u32IDType,rt_uint8_t rtr, rt_uint32_t u32ID);
rt_int32_t CAN_SetTxMsg(struct c_can_priv *can_priv, rt_uint32_t u32MsgNum , struct rt_can_msg* pCanMsg);
rt_int32_t CAN_TriggerTxMsg(struct c_can_priv *can_priv, rt_uint32_t u32MsgNum);
void CAN_ShowMsg(struct rt_can_msg* Msg);

/*@}*/ /* end of group NUC970_CAN_EXPORTED_FUNCTIONS */

/*@}*/ /* end of group NUC970_CAN_Driver */

/*@}*/ /* end of group NUC970_Device_Driver */

#ifdef __cplusplus
} 
#endif

#endif //__CAN_H__

/*** (C) COPYRIGHT 2015 Nuvoton Technology Corp. ***/





